<?php

namespace Ems\Controllers;

use Exception\Messages;
use Service\LogService;
use Service\OAuth\OAuthCustomImpl;
use Zeedhi\Framework\Controller\Simple;
use Zeedhi\Framework\HTTP\Kernel;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\Error;
use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO;

use Ems\Services\ComunicadoAcidenteTrabalhoService;
class ComunicadoAcidenteTrabalhoController {

    /** @var ComunicadoAcidenteTrabalhoService */
    protected $comunicadoAcidenteTrabalhoService;

    public function __construct(
        EntityManager $entityManager,
        ComunicadoAcidenteTrabalhoService $comunicadoAcidenteTrabalhoService
    ) {
        $this->entityManager                     = $entityManager;
        $this->comunicadoAcidenteTrabalhoService = $comunicadoAcidenteTrabalhoService;
    }

    public function getFonteGeradora(
        Request\Row $request,
        Response $response
    ) {
        try {
            $fonteGeradora = $this->comunicadoAcidenteTrabalhoService->getFonteGeradora();
            $response->addDataSet(new DataSet('getFonteGeradora', $fonteGeradora));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

    public function getClassificacaoOcorrencia(
        Request\Row $request,
        Response $response
    ) {
        try {
            $classificacaoOcorrencia = $this->comunicadoAcidenteTrabalhoService->getClassificacaoOcorrencia();
            $response->addDataSet(new DataSet('getClassificacaoOcorrencia', $classificacaoOcorrencia));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

    public function getGravidadeOcorrencia(
        Request\Row $request,
        Response $response
    ) {
        try {
            $gravidadeOcorrencia = $this->comunicadoAcidenteTrabalhoService->getGravidadeOcorrencia();
            $response->addDataSet(new DataSet('getGravidadeOcorrencia', $gravidadeOcorrencia));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

    
    public function getInfluenciaGeral(
        Request\Row $request,
        Response $response
    ) {
        try {
            $influenciaGeral = $this->comunicadoAcidenteTrabalhoService->getInfluenciaGeral();
            $response->addDataSet(new DataSet('getInfluenciaGeral', $influenciaGeral));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

    public function getAgenteCausadorAcidTrab(
        Request\Row $request,
        Response $response
    ) {
        try {
            $agenteCausadorAcidTrab = $this->comunicadoAcidenteTrabalhoService->getAgenteCausadorAcidTrab();
            $response->addDataSet(new DataSet('getAgenteCausadorAcidTrab', $agenteCausadorAcidTrab));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

    public function getNaturezaExame(
        Request\Row $request,
        Response $response
    ) {
        try {
            $naturezaExame = $this->comunicadoAcidenteTrabalhoService->getNaturezaExame();
            $response->addDataSet(new DataSet('getNaturezaExame', $naturezaExame));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

    public function getTipoLocalCat(
        Request\Row $request,
        Response $response
    ) {
        try {
            $tipoLocalCat = $this->comunicadoAcidenteTrabalhoService->getTipoLocalCat();
            $response->addDataSet(new DataSet('getTipoLocalCat', $tipoLocalCat));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

    public function getNaturezaOcorrencia(
        Request\Row $request,
        Response $response
    ) {
        try {
            $naturezaOcorrencia = $this->comunicadoAcidenteTrabalhoService->getNaturezaOcorrencia();
            $response->addDataSet(new DataSet('getNaturezaOcorrencia', $naturezaOcorrencia));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

    public function getAgenteCausador(
        Request\Row $request,
        Response $response
    ) {
        try {
            $agenteCausador = $this->comunicadoAcidenteTrabalhoService->getAgenteCausador();
            $response->addDataSet(new DataSet('getAgenteCausador', $agenteCausador));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

    
    
    public function getParteAtingida(
        Request\Row $request,
        Response $response
    ) {
        try {
            $parteAtingida = $this->comunicadoAcidenteTrabalhoService->getParteAtingida();
            $response->addDataSet(new DataSet('getParteAtingida', $parteAtingida));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

   
    public function getMotivoAusencia(
        Request\Row $request,
        Response $response
    ) {
        try {
            $motivoAusencia = $this->comunicadoAcidenteTrabalhoService->getMotivoAusencia();
            $response->addDataSet(new DataSet('getMotivoAusencia', $motivoAusencia));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }
    

    public function getQueixa(
        Request\Row $request,
        Response $response
    ) {
        try {
            $queixa = $this->comunicadoAcidenteTrabalhoService->getQueixa();
            $response->addDataSet(new DataSet('getQueixa', $queixa));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

    public function getResultadoExame(
        Request\Row $request,
        Response $response
    ) {
        try {
            $resultadoExame = $this->comunicadoAcidenteTrabalhoService->getResultadoExame();
            $response->addDataSet(new DataSet('getResultadoExame', $resultadoExame));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

    public function getSituacao(
        Request\Row $request,
        Response $response
    ) {
        try {
            $situacao = $this->comunicadoAcidenteTrabalhoService->getSituacao();
            $response->addDataSet(new DataSet('getSituacao', $situacao));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

    public function getProximaSituacao(
        Request\Row $request,
        Response $response
    ) {
        try {
            $proximaSituacao = $this->comunicadoAcidenteTrabalhoService->getProximaSituacao();
            $response->addDataSet(new DataSet('getProximaSituacao', $proximaSituacao));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

    public function getCID(
        Request\Row $request,
        Response $response
    ) {
        try {
            $cid = $this->comunicadoAcidenteTrabalhoService->getCID();
            $response->addDataSet(new DataSet('getCID', $cid));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

    public function getControleapt(
        Request\Row $request,
        Response $response
    ) {
        try {
            $controleApt = $this->comunicadoAcidenteTrabalhoService->getControleapt();
            $response->addDataSet(new DataSet('getControleapt', $controleApt));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

    /*

     public function getUnidadeAtendimento(
        Request\Row $request,
        Response $response
    ) {
        try {
            $unidadeAtendimento = $this->comunicadoAcidenteTrabalhoService->getUnidadeAtendimento();
            $response->addDataSet(new DataSet('getUnidadeAtendimento', $unidadeAtendimento));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

    */





    
}