<?php

namespace Ems\Logic\ProcessamentoSolicitacao;

use Ems\Util\Utils;
use Util\{ DateUtil, NovoCodigo, Repositories };

class ProcessamentoSolicitacaoMovimentacaoOcupacaoSalario extends ProcessamentoSolicitacao {
    
    public function efetivar() {
        $itens = $this->getItensAssociativeArray();
        
        $nrorg             = Utils::getParamters()->getNrorg();
        $nrvinculom        = $this->getSolicitacao()->getNrvinculom();
        $dtcompetencia     = DateUtil::getDataDeString(Utils::getParamters()->getDtmescomptencDiaCorrente(), DateUtil::FORMATO_BRASILEIRO,true);
        
        // OCUPAÇÃO
        $nrocupacaom_ocu    = $itens['NROCUPACAOM']     ?? $itens['NROCUPACAOM_OCU'] ?? null; 
        $nrmotivoocupa      = $itens['NRMOTIVOOCUPA']   ?? null; 
        $nrtpmodalidsal     = $itens['NRTPMODALIDSAL']  ?? null; 
        $nrtipoocupacao     = $itens['NRTIPOOCUPACAO']  ?? null;

        // SALÁRIO
        $nrtipoaltera   = $itens['NRTIPOALTERA']        ?? null;
        $idtpsalario    = $itens['IDTPSALARIO']         ?? null;
        $nrocorrencia   = $itens['NROCORRENCIA']        ?? null;
        $novoVlrSalario = $itens['VRSALARIO']           ?? $itens['NOVOVALORSALARIO'] ?? null;

        // Transferência/Remanejamento
        $nrtipotransfer = '1';
        $nrtpmovtransfm = $itens['NRTPMOVTRANSFM']  ?? null;
        $nrmotivotransf = $itens['NRMOTIVOTRANSF']  ?? null;
        $nrestrutlegal  = $itens['NRESTRUTLEGAL']  ?? null;
        $nrestrutgeren  = $itens['NRESTRUTGEREN']  ?? null;
        $dtinimoviment  = isset($itens['DTINIMOVIMENT']) ? DateUtil::getDataDeString(substr($itens['DTINIMOVIMENT'],0,10), DateUtil::FORMATO_BRASILEIRO, true) : null;
        $dtfimmoviment  = isset($itens['DTFIMMOVIMENT']) ? DateUtil::getDataDeString(substr($itens['DTFIMMOVIMENT'],0,10), DateUtil::FORMATO_BRASILEIRO, true) : null;

        $dsobsaltesalario  = $itens['DSOBSALTESALARIO'] ?? '';
        $observacao        = $this->getObservacaoComProtocolo($dsobsaltesalario);
        
        $resultOcupacao = Utils::getEntityManager()->getRepository(Repositories::GPE_ALTEOCUPACAO)->findBy(array("nrorg"=> Utils::getParamters()->getNrorg(),"nrvinculom" => $nrvinculom,"dtiniocupacao" => $dtinimoviment));

        if($resultOcupacao)
            //throw new \Exception("Já existe uma alteração de ocupação para este vinculo nesta data");
        $this->atualizaAlteOcupa($nrvinculom,DateUtil::subtraiIntervalo($dtinimoviment,1,DateUtil::DIA)->format(DateUtil::FORMATO_BRASILEIRO),DateUtil::getDataAtual(true)->format(DateUtil::FORMATO_BRASILEIRO),Utils::getParamters()->getCdoperador());


        $gpeAlteocupacao = Repositories::GPE_ALTEOCUPACAO;
        $alteOcupacao = new $gpeAlteocupacao;
        $alteOcupacao->setNrorg(Utils::getParamters()->getNrorg());
        $alteOcupacao->setNrvinculom($nrvinculom);
        $alteOcupacao->setDsobservacao($observacao);
        $alteOcupacao->setDtiniocupacao($dtinimoviment);
        $alteOcupacao->setNrmotivoocupa($nrmotivoocupa);
        $alteOcupacao->setNrocupacaom($nrocupacaom_ocu);
        $alteOcupacao->setNrtipoocupacao($nrtipoocupacao);
        $alteOcupacao->setNrtpmodalidsal($nrtpmodalidsal);
        $alteOcupacao->setDtinclusao(DateUtil::getDataAtual(false));
        $alteOcupacao->setNrorginclusao(Utils::getParamters()->getNrorg());
        $alteOcupacao->setCdoperinclusao(Utils::getParamters()->getCdoperador());
        $alteOcupacao->setNralteocupacao(NovoCodigo::geraCodigo(Utils::getParamters()->getNrorg(), 'GPE_ALTEOCUPACAO', 1, 12, true));
        Utils::getEntityManager()->persist($alteOcupacao);
        Utils::getEntityManager()->flush();


        // Salário
        if($nrtipoaltera && $novoVlrSalario && $idtpsalario) {
            $alteSalario = ProcessamentoSolicitacaoUtil::novoAlteSalario($nrorg, $nrvinculom, $dtinimoviment, $nrocorrencia, $nrtipoaltera, $nrtpmodalidsal, $idtpsalario, $novoVlrSalario, $observacao);
        }

        
        // Atualiza histórico do vínculo
        $vinc = ProcessamentoSolicitacaoUtil::retornaUltimoHistoricoVinculoCompetencia($nrorg, $nrvinculom, $dtcompetencia);

        if($vinc->getDtmescompetenc() == $dtcompetencia){
            $vinc->setDtultatu(DateUtil::getDataAtual());
            $vinc->setNrorgultatu(Utils::getParamters()->getNrorg());
            $vinc->setCdoperultatu(Utils::getParamters()->getCdoperador());
            $vinc->setNrcargo($nrocupacaom_ocu);

            if($nrestrutlegal) {
                $vinc->setNrestrutlegal($nrestrutlegal);
            }
            if($nrestrutgeren) {
                $vinc->setNrestrutgeren($nrestrutgeren);
            }

            Utils::getEntityManager()->persist($vinc);
            Utils::getEntityManager()->flush();
        }
        else {
            $estrutgeren = $nrestrutgeren ?? $vinc->getNrestrutgeren();
            $estrutlegal = $nrestrutlegal ?? $vinc->getNrestrutlegal();

            ProcessamentoSolicitacaoUtil::novoHistoricoVinculo($nrorg, $nrvinculom, $dtcompetencia, $vinc->getNrsitufuncm(), $nrocupacaom_ocu, $vinc->getNrescalatrabm(),
            $vinc->getIdcontribuisind(), $vinc->getNrdependir(), $vinc->getNrdependsfam(), $vinc->getDtbaseferias(), $vinc->getIdremuneracao(),
            $vinc->getIdtppagamento(), $estrutlegal, $estrutgeren, $vinc->getNrtpmodalidsal(), $vinc->getIdmultvinc(),
            $vinc->getNrfuncao(), $vinc->getCdcontribindividual(), $vinc->getNrtpmovtransfm(), $vinc->getDtfimcontrdetermin(),
            $vinc->getNrestrutsind(), $vinc->getConfidencial()
            );
        }

        
        // Cria a Movimentação Legal
        if($nrestrutlegal) {
            $estrutura_nova_legal = Utils::getEntityManager()->getRepository(Repositories::ESTRUTURAM)->findOneBy(array("nrorg"=> $nrorg,"nrestruturam" => $nrestrutlegal));

            ProcessamentoSolicitacaoUtil::novaMovimentacao($nrorg, $nrvinculom, $estrutura_nova_legal->getNrtipoestrutura(), $nrestrutlegal, $nrmotivotransf, $nrtipotransfer, $nrtpmovtransfm, $dtfimmoviment, $dtinimoviment, $observacao);
            $dtfimmovimentsaida = DateUtil::subtraiIntervalo($dtinimoviment, 1)->format('d/m/Y');
            $dtfimmovimentsaida = DateUtil::getDataDeString(substr($dtfimmovimentsaida,0,10), DateUtil::FORMATO_BRASILEIRO, true);
    
            $estrutura_anterior = Utils::getEntityManager()->getRepository(Repositories::ESTRUTURAM)->findOneBy(array("nrorg"=> $nrorg,"nrestruturam" => $vinc->getNrestrutlegal()));
    
            // Cria a Movimentação de saida
            ProcessamentoSolicitacaoUtil::novaMovimentacao($nrorg, $nrvinculom, $estrutura_anterior->getNrtipoestrutura(), $vinc->getNrestrutlegal(), $nrmotivotransf, $nrtipotransfer, "9", $dtfimmovimentsaida, $dtinimoviment, $observacao);
        }

        // Cria a Movimentação Gerencial
        if($nrestrutgeren) {
            $estrutura_nova_geren = Utils::getEntityManager()->getRepository(Repositories::ESTRUTURAM)->findOneBy(array("nrorg"=> $nrorg,"nrestruturam" => $nrestrutgeren));
            $nrtipotransfergeren = '2';

            ProcessamentoSolicitacaoUtil::novaMovimentacao($nrorg, $nrvinculom, $estrutura_nova_geren->getNrtipoestrutura(), $nrestrutgeren, $nrmotivotransf, $nrtipotransfergeren, $nrtpmovtransfm, $dtfimmoviment, $dtinimoviment, $observacao);
        }

    }
    
    public function rejeitar() {}
    
    public function atualizaAlteOcupa($nrvinculom, $dtfimocupacao,$dtatual,$cdoperador) {
        $sql = <<<SQL
            UPDATE GPE_ALTEOCUPACAO
            SET DTFIMOCUPACAO = :P_DTFIMOCUPACAO,
            DTULTATU = :P_DTATUAL,
            NRORGULTATU = :P_NRORG,
            CDOPERULTATU = :P_CDOPERADOR
            WHERE DTINIOCUPACAO = (
                SELECT MAX(DTINIOCUPACAO) 
                FROM GPE_ALTEOCUPACAO 
                WHERE NRORG = :P_NRORG 
                AND NRVINCULOM = :P_NRVINCULOM 
                AND DTFIMOCUPACAO IS NULL
                )
            AND NRORG = :P_NRORG 
            AND NRVINCULOM = :P_NRVINCULOM 
            AND DTINIOCUPACAO >= :P_DTFIMOCUPACAO
            AND DTFIMOCUPACAO IS NULL
SQL;

        return Utils::getEntityManager()->getConnection()->prepare($sql)
            ->execute(array(
                "P_NRORG"             => Utils::getParamters()->getNrorg(),
                "P_DTFIMOCUPACAO"     => $dtfimocupacao,
                "P_NRVINCULOM"        => $nrvinculom,
                "P_DTATUAL"           => $dtatual,
                "P_CDOPERADOR"        => $cdoperador
            )
        );
    }
}