<?php
namespace Ems\Repository;

use Doctrine\ORM\EntityManager;
use Doctrine\DBAL\Connection;
use Util\NovoCodigo;
use Ems\Util\Utilities;

class ComunicadoAcidenteTrabalhoRepository{
    protected $entityManager;
    protected $connection;

    public function __construct(
        EntityManager $entityManager
    ){
        $this->connection      = $entityManager->getConnection();
        $this->entityManager   = $entityManager;
    }

    public function getFonteGeradora (array $params): array {
        $sql = <<<SQL
        SELECT OC.*,
         SG.DSSITUGERADORA
FROM SST_TIPOOCORRE OC
LEFT JOIN SST_SITUGERADORA SG
    ON SG.NRSITUGERADORA = OC.NRSITUGERADORA
        AND (SG.NRORG = OC.NRORG
        OR SG.NRORG = :P_NRORG_PADRAO)
WHERE OC.NRORG = :P_NRORG
        OR OC.NRORG = :P_NRORG_PADRAO
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $fonteGeradora = $std->fetchAll();

        return $fonteGeradora;
    }

    public function getClassificacaoOcorrencia (array $params): array {
        $sql = <<<SQL
        SELECT CL.*,
         TA.DSTIPOACIDENTE,
         TA.CDESOCIAL
FROM SST_CLASSIFICACAO CL
LEFT JOIN SST_TIPOACIDENTE TA
    ON TA.NRTIPOACIDENTE = CL.NRTIPOACIDENTE
        AND (TA.NRORG = CL.NRORG
        OR TA.NRORG = :P_NRORG_PADRAO)
WHERE CL.NRORG = :P_NRORG
        OR CL.NRORG = :P_NRORG_PADRAO
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        return $std->fetchAll();

        
    }

    public function getGravidadeOcorrencia (array $params): array {
        $sql = <<<SQL
        SELECT *
FROM SST_GRAVIDADE
WHERE (NRORG = :P_NRORG
        OR NRORG = :P_NRORG_PADRAO)
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        return $std->fetchAll();
        
    }
    
    public function getInfluenciaGeral (array $params): array {
        $sql = <<<SQL
  SELECT *
FROM SST_INFLUENCIAGERENCIAL
WHERE (NRORG = :P_NRORG
        OR NRORG = :P_NRORG_PADRAO)
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        return $std->fetchAll();
        
    }

        
    public function getAgenteCausadorAcidTrab (array $params): array {
        $sql = <<<SQL
 SELECT *
FROM SST_CAUSA
WHERE (NRORG = :P_NRORG
        OR NRORG = :P_NRORG_PADRAO)
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        return $std->fetchAll();
        
    }

    public function getNaturezaExame (array $params): array {
        $sql = <<<SQL
 SELECT *
FROM SST_NATUREZAEXAME
WHERE (NRORG = :P_NRORG
        OR NRORG = :P_NRORG_PADRAO)
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        return $std->fetchAll();
        
    }

    public function getTipoLocalCat (array $params): array {
        $sql = <<<SQL
 SELECT NRTIPOLOCALCAT,
         REPLACE(REPLACE(REPLACE(REPLACE(DSTIPOLOCALCAT ,
         'Ãº', 'ú') , 'Ã', 'Á') , 'Ã§', 'ç') , 'Ã£', 'ã') AS DSTIPOLOCALCAT, CDESOCIAL, NRORG, IDATIVO, DTINCLUSAO, DTULTATU, NRORGINCLUSAO, CDOPERINCLUSAO, NRORGULTATU, CDOPERULTATU
FROM SST_TIPOLOCALCAT
WHERE (NRORG = :P_NRORG
        OR NRORG = :P_NRORG_PADRAO)
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        return $std->fetchAll();
        
    }

    public function getNaturezaOcorrencia (array $params): array {
        $sql = <<<SQL
        SELECT SST_NATUREZA.*,
         ESO_NATUREZALESAO.DSNATUREZALESAO
FROM SST_NATUREZA
LEFT JOIN ESO_NATUREZALESAO
    ON ( SST_NATUREZA.NRNATUREZALESAO = ESO_NATUREZALESAO.NRNATUREZALESAO
        AND (ESO_NATUREZALESAO.NRORG = :P_NRORG
        OR ESO_NATUREZALESAO.NRORG = :P_NRORG_PADRAO) )
WHERE (SST_NATUREZA.NRORG = :P_NRORG
        OR SST_NATUREZA.NRORG = :P_NRORG_PADRAO)
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        return $std->fetchAll();
        
    }

    public function getAgenteCausador (array $params): array {
        $sql = <<<SQL
        SELECT *
FROM SST_AGENTCAUSAD
WHERE (NRORG = :P_NRORG
        OR NRORG = :P_NRORG_PADRAO)
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        return $std->fetchAll();
        
    }

    public function getParteAtingida (array $params): array {
        $sql = <<<SQL
        SELECT PA.*,
         PC.DSPARTATINGESOC
FROM SST_PARTEATINGIDA PA
JOIN SST_PARTATINGESOC PC
    ON PC.NRPARTATINGESOC = PA.NRPARTATINGESOC
        AND (PC.NRORG = PA.NRORG
        OR PC.NRORG = :P_NRORG_PADRAO)
WHERE PA.NRORG = :P_NRORG
        OR PA.NRORG = :P_NRORG_PADRAO
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        return $std->fetchAll();
        
    }

    public function getMotivoAusencia (array $params): array {
        $sql = <<<SQL
       SELECT *
FROM SST_MOTIVOAUSENCIA
WHERE (NRORG = :P_NRORG
        OR NRORG = :P_NRORG_PADRAO)
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        return $std->fetchAll();
        
    }


    public function getQueixa (array $params): array {
        $sql = <<<SQL
         SELECT *
FROM SST_QUEIXA
WHERE (NRORG = :P_NRORG
        OR NRORG = :P_NRORG_PADRAO)
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        return $std->fetchAll();
        
    }

    public function getResultadoExame (array $params): array {
        $sql = <<<SQL
        SELECT *
FROM SST_TABRESULTEXAME
WHERE (NRORG = :P_NRORG
        OR NRORG = :P_NRORG_PADRAO)
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        return $std->fetchAll();
        
    }

    public function getSituacao (array $params): array {
        $sql = <<<SQL
        SELECT SM.DTINIVIGENCIA,
         SM.DTFIMVIGENCIA,
         PROX_SITU.NMSITUFUNCH NMPROXSITUFUNC,
         SH.*
FROM GPE_SITUFUNCM SM
JOIN GPE_SITUFUNCH SH
    ON ( SH.NRORG = :P_NRORG
        OR SH.NRORG = :P_NRORG_PADRAO )
        AND SH.NRSITUFUNCM = SM.NRSITUFUNCM
        AND SH.DTEFETIVA = 
    (SELECT MAX(DTEFETIVA)
    FROM GPE_SITUFUNCH
    WHERE NRORG = SH.NRORG
            AND NRSITUFUNCM = SH.NRSITUFUNCM
            AND DTEFETIVA <= :P_DTEFETIVA )
LEFT JOIN GPE_SITUFUNCH PROX_SITU
    ON PROX_SITU.NRSITUFUNCM = SH.NRPROXSITUFUNC
        AND PROX_SITU.DTEFETIVA = 
    (SELECT MAX(DTEFETIVA)
    FROM GPE_SITUFUNCH
    WHERE GPE_SITUFUNCH.NRSITUFUNCM = SH.NRPROXSITUFUNC
            AND GPE_SITUFUNCH.DTEFETIVA <= :P_DTEFETIVA )
WHERE ( SM.NRORG = :P_NRORG
        OR SM.NRORG = :P_NRORG_PADRAO )
        AND ( SM.DTFIMVIGENCIA IS NULL
        OR SM.DTFIMVIGENCIA >= :P_DTEFETIVA
        OR NVL(:P_ATIVO, 'N') != 'S' )
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        return $std->fetchAll();
        
    }

    public function getProximaSituacao (array $params): array {
        $sql = <<<SQL
              SELECT SM.DTINIVIGENCIA,
         SM.DTFIMVIGENCIA,
         PROX_SITU.NMSITUFUNCH NMPROXSITUFUNC,
         SH.*
FROM GPE_SITUFUNCM SM
JOIN GPE_SITUFUNCH SH
    ON ( SH.NRORG = :P_NRORG
        OR SH.NRORG = :P_NRORG_PADRAO )
        AND SH.NRSITUFUNCM = SM.NRSITUFUNCM
        AND SH.DTEFETIVA = 
    (SELECT MAX(DTEFETIVA)
    FROM GPE_SITUFUNCH
    WHERE NRORG = SH.NRORG
            AND NRSITUFUNCM = SH.NRSITUFUNCM
            AND DTEFETIVA <= :P_DTEFETIVA )
LEFT JOIN GPE_SITUFUNCH PROX_SITU
    ON PROX_SITU.NRSITUFUNCM = SH.NRPROXSITUFUNC
        AND PROX_SITU.DTEFETIVA = 
    (SELECT MAX(DTEFETIVA)
    FROM GPE_SITUFUNCH
    WHERE GPE_SITUFUNCH.NRSITUFUNCM = SH.NRPROXSITUFUNC
            AND GPE_SITUFUNCH.DTEFETIVA <= :P_DTEFETIVA )
WHERE ( SM.NRORG = :P_NRORG
        OR SM.NRORG = :P_NRORG_PADRAO )
        AND ( SM.DTFIMVIGENCIA IS NULL
        OR SM.DTFIMVIGENCIA >= :P_DTEFETIVA
        OR NVL(:P_ATIVO, 'N') != 'S' )
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        return $std->fetchAll();
        
    }

    public function getCID (): array {
        $sql = <<<SQL
                SELECT * FROM SST_CID

SQL;

        $std = $this->connection->prepare($sql);
        $std->execute();

        return $std->fetchAll();
        
    }  

    public function getControleapt (array $params): array {
        $sql = <<<SQL
         SELECT *
FROM SST_CONTROLEAPTIDAO
WHERE (NRORG = :P_NRORG
        OR NRORG = :P_NRORG_PADRAO)
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        return $std->fetchAll();
        
    }  
}