<?php

namespace Ems\Services;

use Doctrine\ORM\EntityManager;

use Util\Environment;
use Ems\Repository\ComunicadoAcidenteTrabalhoRepository;

class ComunicadoAcidenteTrabalhoService {
    /** @var ComunicadoAcidenteTrabalhoRepository */
    public $comunicadoAcidenteTrabalhoRepository;

    /** @var Environment */
    public $environment;

    public function __construct(
        EntityManager $entityManager,
        Environment $environment,
        ComunicadoAcidenteTrabalhoRepository $comunicadoAcidenteTrabalhoRepository
    ) {
        $this->entityManager                        = $entityManager;
        $this->comunicadoAcidenteTrabalhoRepository = $comunicadoAcidenteTrabalhoRepository;
        $this->environment                          = $environment;
    }

    public function getFonteGeradora() : array {
        $nrOrg = (int) $this->environment->getNrOrg();
        $nrOrgPadrao = (int) $this->environment->getNrOrgPadrao();

        $params = [
            'P_NRORG_PADRAO'    => $nrOrgPadrao,
            'P_NRORG'           => $nrOrg
        ];

        $fonteGeradora = $this->comunicadoAcidenteTrabalhoRepository->getFonteGeradora($params);

        return $fonteGeradora;
    }

    public function getClassificacaoOcorrencia() : array {
        $nrOrg = (int) $this->environment->getNrOrg();
        $nrOrgPadrao = (int) $this->environment->getNrOrgPadrao();

        $params = [
            'P_NRORG_PADRAO'    => $nrOrgPadrao,
            'P_NRORG'           => $nrOrg
        ];

        $classificacaoOcorrencia = $this->comunicadoAcidenteTrabalhoRepository->getClassificacaoOcorrencia($params);
        return $classificacaoOcorrencia;
    }

    public function getGravidadeOcorrencia() : array {
        $nrOrg = (int) $this->environment->getNrOrg();
        $nrOrgPadrao = (int) $this->environment->getNrOrgPadrao();

        $params = [
            'P_NRORG_PADRAO'    => $nrOrgPadrao,
            'P_NRORG'           => $nrOrg
        ];

        return $this->comunicadoAcidenteTrabalhoRepository->getGravidadeOcorrencia($params);
    }

    public function getInfluenciaGeral() : array {
        $nrOrg = (int) $this->environment->getNrOrg();
        $nrOrgPadrao = (int) $this->environment->getNrOrgPadrao();

        $params = [
            'P_NRORG_PADRAO'    => $nrOrgPadrao,
            'P_NRORG'           => $nrOrg
        ];

        return $this->comunicadoAcidenteTrabalhoRepository->getInfluenciaGeral($params);
    }

    public function getAgenteCausadorAcidTrab() : array {
        $nrOrg = (int) $this->environment->getNrOrg();
        $nrOrgPadrao = (int) $this->environment->getNrOrgPadrao();

        $params = [
            'P_NRORG_PADRAO'    => $nrOrgPadrao,
            'P_NRORG'           => $nrOrg
        ];

        return $this->comunicadoAcidenteTrabalhoRepository->getAgenteCausadorAcidTrab($params);
    }

    public function getNaturezaExame() : array {
        $nrOrg = (int) $this->environment->getNrOrg();
        $nrOrgPadrao = (int) $this->environment->getNrOrgPadrao();

        $params = [
            'P_NRORG_PADRAO'    => $nrOrgPadrao,
            'P_NRORG'           => $nrOrg
        ];

        return $this->comunicadoAcidenteTrabalhoRepository->getNaturezaExame($params);
    }

    public function getTipoLocalCat() : array {
        $nrOrg = (int) $this->environment->getNrOrg();
        $nrOrgPadrao = (int) $this->environment->getNrOrgPadrao();

        $params = [
            'P_NRORG_PADRAO'    => $nrOrgPadrao,
            'P_NRORG'           => $nrOrg
        ];

        return $this->comunicadoAcidenteTrabalhoRepository->getTipoLocalCat($params);
    }

    public function getNaturezaOcorrencia() : array {
        $nrOrg = (int) $this->environment->getNrOrg();
        $nrOrgPadrao = (int) $this->environment->getNrOrgPadrao();

        $params = [
            'P_NRORG_PADRAO'    => $nrOrgPadrao,
            'P_NRORG'           => $nrOrg
        ];

        return $this->comunicadoAcidenteTrabalhoRepository->getNaturezaOcorrencia($params);
    }

    public function getAgenteCausador() : array {
        $nrOrg = (int) $this->environment->getNrOrg();
        $nrOrgPadrao = (int) $this->environment->getNrOrgPadrao();

        $params = [
            'P_NRORG_PADRAO'    => $nrOrgPadrao,
            'P_NRORG'           => $nrOrg
        ];

        return $this->comunicadoAcidenteTrabalhoRepository->getAgenteCausador($params);
    }

    public function getParteAtingida() : array {
        $nrOrg = (int) $this->environment->getNrOrg();
        $nrOrgPadrao = (int) $this->environment->getNrOrgPadrao();

        $params = [
            'P_NRORG_PADRAO'    => $nrOrgPadrao,
            'P_NRORG'           => $nrOrg
        ];

        return $this->comunicadoAcidenteTrabalhoRepository->getParteAtingida($params);
    }

    public function getMotivoAusencia() : array {
        $nrOrg = (int) $this->environment->getNrOrg();
        $nrOrgPadrao = (int) $this->environment->getNrOrgPadrao();

        $params = [
            'P_NRORG_PADRAO'    => $nrOrgPadrao,
            'P_NRORG'           => $nrOrg
        ];

        return $this->comunicadoAcidenteTrabalhoRepository->getMotivoAusencia($params);
    }

    public function getQueixa() : array {
        $nrOrg = (int) $this->environment->getNrOrg();
        $nrOrgPadrao = (int) $this->environment->getNrOrgPadrao();

        $params = [
            'P_NRORG_PADRAO'    => $nrOrgPadrao,
            'P_NRORG'           => $nrOrg
        ];

        return $this->comunicadoAcidenteTrabalhoRepository->getQueixa($params);
    }

    public function getResultadoExame() : array {
        $nrOrg = (int) $this->environment->getNrOrg();
        $nrOrgPadrao = (int) $this->environment->getNrOrgPadrao();

        $params = [
            'P_NRORG_PADRAO'    => $nrOrgPadrao,
            'P_NRORG'           => $nrOrg
        ];

        return $this->comunicadoAcidenteTrabalhoRepository->getResultadoExame($params);
    }

    public function getSituacao() : array {
        $nrOrg = (int) $this->environment->getNrOrg();
        $nrOrgPadrao = (int) $this->environment->getNrOrgPadrao();
        $dtMesCompetencia = $this->environment->getDtmescomptencDiaCorrente();

        $params = [
            'P_NRORG_PADRAO'    => $nrOrgPadrao,
            'P_NRORG'           => $nrOrg,
            'P_DTEFETIVA'       => $dtMesCompetencia,
            'P_ATIVO'           => 'S'
        ];

        return $this->comunicadoAcidenteTrabalhoRepository->getSituacao($params);
    }

    public function getProximaSituacao() : array {
        $nrOrg = (int) $this->environment->getNrOrg();
        $nrOrgPadrao = (int) $this->environment->getNrOrgPadrao();
        $dtMesCompetencia = $this->environment->getDtmescomptencDiaCorrente();

        $params = [
            'P_NRORG_PADRAO'    => $nrOrgPadrao,
            'P_NRORG'           => $nrOrg,
            'P_DTEFETIVA'       => $dtMesCompetencia,
            'P_ATIVO'           => 'S'
        ];

        return $this->comunicadoAcidenteTrabalhoRepository->getProximaSituacao($params);
    }

    public function getCID() : array {

        return $this->comunicadoAcidenteTrabalhoRepository->getCID();
    }  

    public function getControleapt() : array {
        $nrOrg = (int) $this->environment->getNrOrg();
        $nrOrgPadrao = (int) $this->environment->getNrOrgPadrao();

        $params = [
            'P_NRORG_PADRAO'    => $nrOrgPadrao,
            'P_NRORG'           => $nrOrg
        ];

        return $this->comunicadoAcidenteTrabalhoRepository->getControleapt($params);
    }  
}