<?php

namespace Eso\Controllers;

use Doctrine\ORM\EntityManager;
use Eso\Logic\ESocialLogic;
use HCM\Util\Repositories;
use Util\DateUtil;
use Util\Environment;
use Util\HcmServices;
use Util\Messages;
use Util\Util;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;

class EsoSimplificadoController {

    public function __construct (EntityManager $entityManager, Environment $environment, HcmServices $hcmservices){
        $this->entityManager = $entityManager;
        $this->environment = $environment;
        $this->hcmservices = $hcmservices;
    }
    
    private function callProcessaFilaPeriodico($nrorg, $cdoperador, $dtmescompetenc, $nrestruturam, $nrtpambiente, $nmevento, $nresoconsumer, $nrlote, $nrenvio, $countProc) {
        $uri = 'processaFilaPeriodico';
        $params = array(
            'requestType' => 'Row',
            'row' => array(
                "NRORG" => $nrorg,
                "CDOPERADOR" => $cdoperador,
                "DTMESCOMPETENC" => $dtmescompetenc,
                "NRESTRUTURAM" => $nrestruturam,
                "NRTPAMBIENTE" => $nrtpambiente,
                "NMEVENTO" => $nmevento,
                "NRESOCONSUMER" => $nresoconsumer,
                "COUNTPROC" => $countProc,
            )
        );
        try {
            Util::writeLogFile($this->environment->getNrOrgTrab(), "callProcessaFilaPeriodico$countProc (Lote:$nrlote Envio: $nrenvio)->processaFilaPeriodico","consumer".$nresoconsumer);
            $result = $this->hcmservices->sendAsync($uri, $params, $nrorg);
        } catch (\Exception $e) {
            if (strpos($e->getMessage(), 'cURL error 28') === false) {
                Util::writeLogFile($this->environment->getNrOrgTrab(), "error callProcessaFilaPeriodico$countProc ->processaFilaPeriodico [Code:".$e->getCode().' Message:'.$e->getMessage().']',"consumer".$nresoconsumer);
                if ($e->getCode() == 502 || $e->getCode() == 500) { //500 Internal Server Error
                    usleep(100000);
                    $this->callProcessaFilaPeriodico($nrorg, $cdoperador, $dtmescompetenc, $nrestruturam, $nrtpambiente, $nmevento, $nresoconsumer, $nrlote, $nrenvio, $countProc);
                } else {
                    throw $e;
                }
            } else {
                if (strpos($e->getMessage(), 'Resolving timed out after') === true) {
                    usleep(100000);
					Util::writeLogFile($this->environment->getNrOrgTrab(), "catch callProcessaFilaPeriodico$countProc ->callProcessaFilaPeriodico (Lote:$nrlote Envio: $nrenvio) [Code:" . $e->getCode() . " Message:" . $e->getMessage() . "]", "consumer$nresoconsumer");
                    $this->callProcessaFilaPeriodico($nrorg, $cdoperador, $dtmescompetenc, $nrestruturam, $nrtpambiente, $nmevento, $nresoconsumer, $nrlote, $nrenvio, $countProc);
                }
            }
        }
    }
    
    private function callEnviarLoteEventoPeriodico($nrorg, $cdoperador, $dtmescompetenc, $nrestruturam, $nrtpambiente, $nmevento, $nresoconsumer, $nrlote, $eventos, $countProc) {
        $uri = 'enviarLoteEventoPeriodico';
        $params = array(
            'requestType' => 'Row',
            'row' => array(
                "NRORG" => $nrorg,
                "CDOPERADOR" => $cdoperador,
                "DTMESCOMPETENC" => $dtmescompetenc,
                "NRESTRUTURAM" => $nrestruturam,
                "NRTPAMBIENTE" => $nrtpambiente,
                "NMEVENTO" => $nmevento,
                "NRESOCONSUMER" => $nresoconsumer,
                "NRLOTE" => $nrlote,
                "EVENTOS" => $eventos,
                "COUNTPROC" => $countProc,
            )
        );
        
        $nrenvio = $eventos[0];
        
        try {
            Util::writeLogFile($this->environment->getNrOrgTrab(), "callEnviarLoteEventoPeriodico$countProc ->enviarLoteEventoPeriodico (Lote:$nrlote Eventos:".implode(',', $eventos).")","consumer$nresoconsumer");
            $result = $this->hcmservices->sendAsync($uri, $params, $nrorg);
        } catch (\Exception $e) {
            if (strpos($e->getMessage(), 'cURL error 28') === false) {
                Util::writeLogFile($this->environment->getNrOrgTrab(), "error callEnviarLoteEventoPeriodico$countProc ->enviarLoteEventoPeriodico [Code:" . $e->getCode() . " Message:" . $e->getMessage() . "]", "consumer$nresoconsumer");
                if ($e->getCode() == 502 || $e->getCode() == 500 || $e->getCode() == 50) { //500 Internal Server Error / 50 Erro de comunicação com o eSocial
                    Util::writeLogFile($this->environment->getNrOrgTrab(), "error callEnviarLoteEventoPeriodico$countProc ->callEnviarLoteEventoPeriodico (Lote:$nrlote Eventos:".implode(',', $eventos).")","consumer$nresoconsumer");
                    $this->callEnviarLoteEventoPeriodico($nrorg, $cdoperador, $dtmescompetenc, $nrestruturam, $nrtpambiente, $nmevento, $nresoconsumer, $nrlote, $eventos, $countProc);
                } else {
                    Util::writeLogFile($this->environment->getNrOrgTrab(), "error callEnviarLoteEventoPeriodico$countProc status erro (Lote:$nrlote Envio: $nrenvio","consumer$nresoconsumer");
                    $this->entityManager->getRepository(Repositories::ESO_LOTE)->atualizaStatusLoteErro($nrorg, $cdoperador, $nresoconsumer, $nrlote, $nrenvio);
                    throw $e;
                }
            } else {
                if (strpos($e->getMessage(), 'Resolving timed out after') === true) {
                    Util::writeLogFile($this->environment->getNrOrgTrab(), "error callEnviarLoteEventoPeriodico$countProc ->enviarLoteEventoPeriodico [Code:" . $e->getCode() . " Message:" . $e->getMessage() . "]", "consumer$nresoconsumer");
                    Util::writeLogFile($this->environment->getNrOrgTrab(), "error callEnviarLoteEventoPeriodico$countProc ->callEnviarLoteEventoPeriodico (Lote:$nrlote Eventos:".implode(',', $eventos).")","consumer$nresoconsumer");
                    $this->callEnviarLoteEventoPeriodico($nrorg, $cdoperador, $dtmescompetenc, $nrestruturam, $nrtpambiente, $nmevento, $nresoconsumer, $nrlote, $eventos, $countProc);
                }
            }
        }
    }

    public function processaFilaPeriodico(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $eventos = array();

            $row = $request->getRow();

            $nrorg = $row['NRORG'];
            $cdoperador = $row['CDOPERADOR'];
            $dtmescompetenc = $row['DTMESCOMPETENC'];
            $nrestruturam = $row["NRESTRUTURAM"];
            $nrtpambiente = $row["NRTPAMBIENTE"];
            $nmevento = $row['NMEVENTO'];
            $nresoconsumer = $row['NRESOCONSUMER'];
            $countProc = isset($row['COUNTPROC']) ? $row['COUNTPROC'] : 0;
            
            $countProc++;
            if ($countProc == 50) {
                sleep(5);
                $countProc = 0;
            }

            $this->environment->setCdOperador($cdoperador);
            $this->environment->setDtCompetencia($dtmescompetenc);

            $logfile = Util::writeLogFile($this->environment->getNrOrgTrab(), "start processaFilaPeriodico$countProc ","consumer$nresoconsumer");
            $this->entityManager->getRepository(Repositories::ESO_CONSUMER)->addLogFile($nrorg, $nresoconsumer, $logfile);

            $lote = $this->entityManager->getRepository(Repositories::ESO_LOTE)->getProximoLoteFila($nrorg, $nresoconsumer);
            
            if ($lote) {
                $nrlote = $lote['NRLOTE'];
                $nrenvio = $lote['NRENVIO'];

                $eventos[] = $nrenvio;

                /*$envios = $this->getEventosLote($nrorg, $nresoconsumer, $nrlote);
                foreach ($envios as $envio) {
                    $eventos[] = $envio['NRENVIO'];
                }*/

                // Chama o próprio Hcmservices
                Util::writeLogFile($this->environment->getNrOrgTrab(), "processaFilaPeriodico$countProc ->callEnviarLoteEventoPeriodico (Lote:$nrlote Eventos:".implode(',', $eventos).")","consumer$nresoconsumer");
                $this->callEnviarLoteEventoPeriodico($nrorg, $cdoperador, $dtmescompetenc, $nrestruturam, $nrtpambiente, $nmevento, $nresoconsumer, $nrlote, $eventos, $countProc);

                // Chama o próprio Hcmservices para continuar a fila
                usleep(100000);
				Util::writeLogFile($this->environment->getNrOrgTrab(), "processaFilaPeriodico$countProc ->callProcessaFilaPeriodico (Lote:$nrlote Eventos:".implode(',', $eventos).")","consumer$nresoconsumer");
                $this->callProcessaFilaPeriodico($row["NRORG"], $row["CDOPERADOR"], $row["DTMESCOMPETENC"], $row["NRESTRUTURAM"], $row["NRTPAMBIENTE"], $row["NMEVENTO"], $row["NRESOCONSUMER"], $nrlote, $nrenvio, $countProc);

                Util::writeLogFile($this->environment->getNrOrgTrab(), "finished processaFilaPeriodico$countProc","consumer".$nresoconsumer);
            } else {
                Util::writeLogFile($this->environment->getNrOrgTrab(), "processaFilaPeriodico$countProc conclui Consumidor","consumer".$nresoconsumer);
                $this->entityManager->getRepository(Repositories::ESO_CONSUMER)->concluiEsoConsumer($nrorg, $nresoconsumer);
            }
        } catch(\Exception $e) {
            Util::writeLogFile($this->environment->getNrOrgTrab(), "error processaFilaPeriodico$countProc [".$e->getCode().' => '.$e->getMessage().']',"consumer".$nresoconsumer);
            if ($e->getCode() == 502 || $e->getCode() == 500) { //500 Internal Server Error
                // Chama o próprio Hcmservices para continuar a fila
                usleep(100000);
				Util::writeLogFile($this->environment->getNrOrgTrab(), "error processaFilaPeriodico$countProc ->callProcessaFilaPeriodico (Lote:$nrlote Eventos:".implode(',', $eventos).")","consumer$nresoconsumer");
                $this->callProcessaFilaPeriodico($row["NRORG"], $row["CDOPERADOR"], $row["DTMESCOMPETENC"], $row["NRESTRUTURAM"], $row["NRTPAMBIENTE"], $row["NMEVENTO"], $row["NRESOCONSUMER"], $nrlote, $nrenvio, $countProc);
            } else if ($e->getCode() == Messages::ALERT_MESSAGE_CODE || $e->getCode() == Messages::ERROR_MESSAGE_CODE) {
                $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
            } else {
                $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
            }
        }
    }

    /**
     * @param Request\Row $request
     * @param DTO\Response $response
     */
    public function enviarLoteEventoPeriodico(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $msgValidacao = array();
            $msgRetorno = "";
            
            $row = $request->getRow();

            $nrorg = $row['NRORG'];
            $cdoperador = $row['CDOPERADOR'];
            $dtmescompetenc = $row['DTMESCOMPETENC'];
            $nrestruturam = $row["NRESTRUTURAM"];
            $nrtpambiente = $row["NRTPAMBIENTE"];
            $nresoconsumer = $row['NRESOCONSUMER'];
            $nmevento = $row['NMEVENTO'];
            $nrlote = $row['NRLOTE'];
            $eventos = $row['EVENTOS'];
            $countProc = $row['COUNTPROC'];

            $lote = current($this->entityManager->getRepository(Repositories::ESO_LOTE)->getEventosLote($nrorg, $nresoconsumer, $nrlote));

            if ($lote['IDSITUACAO'] == 'PENDENTE') {

                $logfile = Util::writeLogFile($this->environment->getNrOrgTrab(), "start enviarLoteEventoPeriodico$countProc (Lote:$nrlote)", "consumer" . $nresoconsumer);

                Util::writeLogFile($this->environment->getNrOrgTrab(), "enviarLoteEventoPeriodico$countProc status processamento (Lote:$nrlote)", "consumer$nresoconsumer");
                $this->entityManager->getRepository(Repositories::ESO_LOTE)->atualizaStatusLoteProcessamento($nrorg, $cdoperador, $nresoconsumer, $nrlote, $eventos[0]);

                $this->environment->setCdOperador($cdoperador);
                $this->environment->setDtCompetencia($dtmescompetenc);

                $logic = new ESocialLogic($this->entityManager, $this->environment, $dtmescompetenc, $row['NRESTRUTURAM'], $row['NRTPAMBIENTE']);
                $msgRetorno = $logic->preparaEnviarLoteEvento(ESocialLogic::IDGRUPO_PERIODICO, $eventos, $nrlote, $nresoconsumer, $logfile);
                $msgValidacao = $logic->getMessages();

                Util::writeLogFile($this->environment->getNrOrgTrab(), "status concluido (Lote:$nrlote)", "consumer$nresoconsumer");
                $this->entityManager->getRepository(Repositories::ESO_LOTE)->atualizaStatusLoteConcluido($nrorg, $cdoperador, $nresoconsumer, $nrlote);

                if (count($msgValidacao) > 0) {
                    Util::writeLogFile($this->environment->getNrOrgTrab(), "finished enviarLoteEventoPeriodico$countProc msgValidacao (Lote:$nrlote)", "consumer" . $nresoconsumer);
                    $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('enviarLoteEventoPeriodico', $msgValidacao));
                } elseif ($msgRetorno) {
                    Util::writeLogFile($this->environment->getNrOrgTrab(), "finished enviarLoteEventoPeriodico$countProc $msgRetorno (Lote:$nrlote) $msgRetorno", "consumer" . $nresoconsumer);
                    $response->addMessage(new DTO\Response\Message($msgRetorno));
                } else {
                    Util::writeLogFile($this->environment->getNrOrgTrab(), "finished enviarLoteEventoPeriodico$countProc (Lote:$nrlote) Envio de evento não disponível", "consumer" . $nresoconsumer);
                    $response->addMessage(new DTO\Response\Message("Envio de evento não disponível", DTO\Response\Message::TYPE_WARNING));
                }

            }
        } catch(\Exception $e) {
            Util::writeLogFile($this->environment->getNrOrgTrab(), "error enviarLoteEventoPeriodico$countProc (Lote:$nrlote) (".$e->getCode().'-'.$e->getMessage().')',"consumer".$nresoconsumer);
            if ($e->getCode() != 502 && $e->getCode() != 500 && $e->getCode() != 50) {
                foreach ($eventos as $nrenvio) {
                    Util::writeLogFile($this->environment->getNrOrgTrab(), "error enviarLoteEventoPeriodico$countProc status erro (Lote:$nrlote Envio: $nrenvio","consumer$nresoconsumer");
                    $this->entityManager->getRepository(Repositories::ESO_LOTE)->atualizaStatusLoteErro($nrorg, $cdoperador, $nresoconsumer, $nrlote, $nrenvio);
                }
                //$response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
            } else {
                foreach ($eventos as $nrenvio) {
                    Util::writeLogFile($this->environment->getNrOrgTrab(), "error enviarLoteEventoPeriodico$countProc status erro (Lote:$nrlote Envio: $nrenvio","consumer$nresoconsumer");
                    $this->entityManager->getRepository(Repositories::ESO_LOTE)->atualizaStatusLoteErro($nrorg, $cdoperador, $nresoconsumer, $nrlote, $nrenvio);
                    
                    Util::writeLogFile($this->environment->getNrOrgTrab(), "error enviarLoteEventoPeriodico enfileira Envio (Lote:$nrlote Envio: $nrenvio)","consumer$nresoconsumer");
                    $this->entityManager->getRepository(Repositories::ESO_LOTE)->enfileiraEnvio($nrorg, $nrlote, $nresoconsumer, $nrenvio, $nmevento, $cdoperador);
                    
                    usleep(100000);
					Util::writeLogFile($this->environment->getNrOrgTrab(), "error enviarLoteEventoPeriodico$countProc callProcessaFilaPeriodico (Lote:$nrlote Envio: $nrenvio)","consumer$nresoconsumer");
                    $this->callProcessaFilaPeriodico($nrorg, $cdoperador, $dtmescompetenc, $nrestruturam, $nrtpambiente, $nmevento, $nresoconsumer, $nrlote, $nrenvio, $countProc);
                }
                //$response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
            }
        }
    }
    
    public function consultarProtocoloPeriodico(DTO\Request\Row $request, DTO\Response $response) {
        try {
            
            $row = $request->getRow();
            
            $this->environment->setCdOperador($row['CDOPERADOR']);
            $this->environment->setDtCompetencia($row['DTMESCOMPETENC']);
                
            $logic = new ESocialLogic($this->entityManager, $this->environment, $row['DTMESCOMPETENC'], $row['NRESTRUTURAM'], $row['NRTPAMBIENTE']);
            $logfile = Util::writeLogFile($this->environment->getNrOrgTrab(), 'called consultarProtocoloPeriodico',"consumer".$row['NRESOCONSUMER']);
            $msgRetorno = $logic->preparaConsultarLoteEvento(ESocialLogic::IDGRUPO_PERIODICO, $row, $row['NRESOCONSUMER'], $logfile);
            $msgValidacao = $logic->getMessages();
            
            if ($msgValidacao) {
                $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('consultarProtocoloPeriodico', array($msgValidacao)));
            } elseif ($msgRetorno) {
                $response->addMessage(new DTO\Response\Message($msgRetorno));
            } else {
                $response->addMessage(new DTO\Response\Message("Consulta de evento não disponível", DTO\Response\Message::TYPE_WARNING));
            }

            $logfile = Util::writeLogFile($this->environment->getNrOrgTrab(), 'finished consultarProtocoloPeriodico',"consumer".$row['NRESOCONSUMER']);
        } catch(\Exception $e) {
            Util::writeLogFile($this->environment->getNrOrgTrab(), 'error consultarProtocoloPeriodico ('.$e->getMessage().')',"consumer".$row['NRESOCONSUMER']);
            if ($e->getCode() == Messages::ALERT_MESSAGE_CODE || $e->getCode() == Messages::ERROR_MESSAGE_CODE) {
                $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
            } else {
                $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
            }
        }
    }
}