<?php

namespace HCM\FPA\Services;

use Zeedhi\Framework\HTTP\Kernel;
use Doctrine\ORM\EntityManager;
use Teknisa\Entities\Contcorrparc;
use Geral\Entities\Agencia;
use HCM\Util\DateUtil;
use HCM\Util\Repositories;
use Geral\Util\NovoCodigo;
use HCM\Util\HCMParameters;
use HCM\Util\Messages;
use HCM\Util\FPA;

class ImportacaoRemessaCcBbService {

    protected $hcmParameters;

    public function __construct(Kernel $httpKernel, EntityManager $entityManager, HCMParameters $hcmParameters) {
        $this->httpKernel              = $httpKernel;          
        $this->entityManager           = $entityManager;
        $this->hcmParameters           = $hcmParameters;
        $this->contabancaria = Repositories::CONTCORRPARC;
        $this->agencia = Repositories::AGENCIA;
    }

    private static $erroCliente = array( 
        '001' => "Tipo de pessoa inválido",
        '002' => "tipo CPF/CNPJ inválido",
        '003' => "CPF/CNPJ inválido",
        '004' => "data nascimento invalida",
        '005' => "nome cliente inválido",
        '006' => "agência/dv inválido",
        '007' => "mais de 5 clientes cadastrados para CPF informado",
        '008' => "cliente BBCampus fora da faixa etária (16 a 28 anos)",
        '009' => "cliente BBCampus não é pessoa física",
        '010' => "dados pessoa física divergentes",
        '011' => "dados pessoa jurídica divergentes",
        '013' => "cliente BBCampus não titular CPF",
        '014' => "perfil agência incompatível com tipo pessoa"
    );
    
    private static $erroConta = array( 
        '001' => "ind cheque especial inválido",
        '002' => "setex/dv inválido",
        '003' => "nao atende ao credit scoring",
        '004' => "dados pessoa física divergentes"
    );
    
    private static $erroCredito = array( 
        '001' => "cod estado civil inválido",
        '002' => "cod natureza ocupação inválido",
        '003' => "cod ocupação inválido",
        '004' => "valor rendimento inválido",
        '005' => "data rendimento invalida",
        '006' => "tipo contrato trabalho inválido",
        '007' => "data início emprego inválida",
        '008' => "nao atende ao credit scoring"
    );
    
    public function existeContParc($cdcontcorr, $nrparcnegocio, $nrorg, $cdagencia) {
        try {
            $params = array(
                'P_CDCONTCORR'      => $cdcontcorr,
                'P_NRPARCNEGOCIO'   => $nrparcnegocio,
                'P_NRORG'           => $nrorg,
                'P_CDAGENCIA'           => $cdagencia);
            $where = "NRORG = :P_NRORG AND CDCONTCORR = :P_CDCONTCORR AND NRPARCNEGOCIO = :P_NRPARCNEGOCIO AND CDAGENCIA = :P_CDAGENCIA";
            $contcorr = Contcorrparc::getObjectsByWhere($where, $params);
            if($contcorr){
                return $contcorr;
            }
            return false;
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function existeAgencia($nrorg, $cdagencia) {
        try {
            $params = array(
                'P_NRORG'           => $nrorg,
                'P_CDAGENCIA'           => $cdagencia);
            $where = "(NRORG = :P_NRORG OR NRORG = 0) AND CDAGENCIA = :P_CDAGENCIA AND CDBANCO = '001'";
            $agencia = Agencia::getObjectsByWhere($where, $params);
            if($agencia){
                return $agencia;
            }
            return false;
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function ImportacaoRemessaCcBb($row){
        try {
            $dtmescompetenc = $this->hcmParameters->getDtCompetencia();
            $nrorg = $this->hcmParameters->getNrorgTrabalho(); 
            $arquivo = $row['DSPATH'][0];
            $message = array();
            $this->entityManager->beginTransaction();
            $file = $this->httpKernel->factoryFromFileData($arquivo, FPA::getTempDirectory() . '/');
            if (!$file->isFile()) {
                throw new \Exception("Falha ao ler o arquivo.", Messages::ALERT_MESSAGE_CODE);
            }
            $arquivo = file_get_contents($file->getPathname());
            $arquivo = str_replace("\r", '', $arquivo);
            $arquivo = explode("\n", $arquivo);
            
            
            //cabeçalho $arquivo[0]
            if(substr($arquivo[0], 0, 5) != '00000'){//digitos de validação
                throw new \Exception("Arquivo Inválido!", Messages::ALERT_MESSAGE_CODE);
            }
            
            if(substr($arquivo[0], 31, 2) != '01'){ //versão do layout
                throw new \Exception("Versão do layout inválida!", Messages::ALERT_MESSAGE_CODE);
            }
            
            $dtarquivo  = substr($arquivo[0], 6, 8);  //data do arquivo
            $nrprocesso = substr($arquivo[0], 21, 5); //número do processo
            $sequencial = substr($arquivo[0], 26, 5); //sequencial de remessa
            
            unset($arquivo[0]); //remove cabeçalho do array
            unset($arquivo[sizeof($arquivo)]); //remove trailer do array
            
            $registros = array();
            foreach ($arquivo as $detalhe) { //percorre registros de conteúdo
                $aux = array();
                $aux['nome']     = rtrim(substr($detalhe, 27, 60));
                $aux['agencia']  = substr($detalhe, 104, 5);  // agencia 104-107 + dv-agencia 108
                $aux['conta']    = substr($detalhe, 112, 12); // conta 112-122 + dv-conta 123
                $aux['erroCli']  = substr($detalhe, 124, 3);  // código de erro do clente - 000 = sem erro
                $aux['erroCon']  = substr($detalhe, 127, 3);  // código de erro da conta
                $aux['erroCre']  = substr($detalhe, 130, 3);  // código de erro do limite de crédito
                
                $registros[rtrim(substr($detalhe, 87, 17))] = $aux; // texto de uso da empresa -> nrparcnegocio 
            }
            
            foreach ($registros as $nrparcnegocio => $registro) {
                $erro = false;
                if(isset(self::$erroCliente[$registro['erroCli']])){
                    $message[]['MENSAGEM'] = "Erro - ".$registro['nome']." - ".self::$erroCliente[$registro['erroCli']];
                    $erro = true;
                }
                if(isset(self::$erroConta[$registro['erroCon']])){
                    $message[]['MENSAGEM'] = "Erro - ".$registro['nome']." - ".self::$erroConta[$registro['erroCon']];
                    $erro = true;
                }
                if(isset(self::$erroCredito[$registro['erroCre']])){
                    $message[]['MENSAGEM'] = "Erro - ".$registro['nome']." - ".self::$erroCredito[$registro['erroCre']];
                    $erro = true;
                }
                if($erro)
                    unset($registros[$nrparcnegocio]);
                
            }
            
            foreach ($registros as $nrparcnegocio => $registro) {
                
                if ($this->existeContParc($registro['conta'], $nrparcnegocio, $nrorg, $registro['agencia'])){
                    $message[]['MENSAGEM'] = "Sucesso - ".$registro['nome']." - Já cadastrado anteriormente.";
                    continue;
                }
                $obj = new $this->contabancaria;
            
                $obj->setIdativo("S");
                $obj->setNrorg($nrorg);
                $obj->setNrorginclusao($nrorg);
                $obj->setCdoperinclusao($this->hcmParameters->getCdoperador());
                $obj->setCdoperultatu($this->hcmParameters->getCdoperador());
                $obj->setDtinclusao(DateUtil::getDataAtual());
                $obj->setDtultatu(DateUtil::getDataAtual());
                $obj->setNrcontcorrparc(\HCM\Util\NovoCodigo::geraCodigo($nrorg, 'CONTCORRPARC', 1, 12, true));
                
                $obj->setCdagencia($registro['agencia']);
                $obj->setCdcontcorr($registro['conta']);
                $obj->setNrparcnegocio($nrparcnegocio);
                $obj->setNrtpcontcorr(2);
                $obj->setCdbanco('001');
                
                if($this->existeAgencia($nrorg, $registro['agencia'])){
                    $message[]['MENSAGEM'] = "Sucesso - ".$registro['nome'];
                }else {
                    $message[]['MENSAGEM'] = "Alerta - ".$registro['nome']." - Agência ".$registro['agencia']." não cadastrada!";
                }
                $this->entityManager->persist($obj);
                $this->entityManager->flush($obj);
    
            }
            
            $this->entityManager->commit();
            return $message;
        
            
        } catch (\Exception $e) {
            $this->entityManager->rollback();
            throw $e;
        }
    }
    

}