<?php
namespace HCM\Geral\Repository\Esocial;

use Doctrine\DBAL\Connection;
use Doctrine\ORM\EntityManager;
use HCM\Util\HCMParameters;

class S2221Repository {

    protected  $connection;

    public function __construct(EntityManager $entityManager, HCMParameters $hcmParameters) {
        $this->connection = $entityManager->getConnection();
        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;
    }
    
    public function getItensParaEnvioS2221($params) {
        $sql = <<<SQL
/*getItensParaEnvioS2221*/
SELECT EFM.NREXAMEFICHAMED as NRCHAVE,
       VH.NRVINCULOM,
       '' as NRTABELA,
       DECODE(ENV.ENVIOS,NULL,'I','A') as IDOPERACAO,
       NVL(EFM.DTREALIZACAOEXAME,FM.DTCONCLFICHAMED) as DTMESCOMPETENC
  FROM SST_EXAMEFICHAMED EFM
  JOIN SST_EXAME EXA ON EXA.NRORG = EFM.NRORG
                    AND EXA.NREXAME = EFM.NREXAME
  JOIN SST_FICHAMEDICA FM ON FM.NRORG = EFM.NRORG
                         AND FM.NRFICHAMEDICA = EFM.NRFICHAMEDICA
  JOIN GPE_VINCULOM VM ON VM.NRORG = FM.NRORG
                      AND VM.NRVINCULOM = FM.NRVINCULOM
  JOIN GPE_VINCULOH VH ON VH.NRORG = VM.NRORG
                      AND VH.NRVINCULOM = VM.NRVINCULOM
                      AND VH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_VINCULOH H1
                                                WHERE H1.NRVINCULOM = VH.NRVINCULOM
                                                  AND H1.NRORG = VH.NRORG
                                                  AND H1.DTMESCOMPETENC >= ADD_MONTHS(TRUNC(TO_DATE(:P_DTMESCOMPETENC),'YEAR'),-12)
                                                  AND H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC)
  /*QUERY PARA VERIFICAR SE É INCLUSÃO OU ALTERAÇÃO DO REGISTRO*/
  LEFT JOIN (SELECT EC.NRORG, EC.NRCHAVE, COUNT(*) AS ENVIOS
               FROM ESO_ENVIOCADASTROS EC
               JOIN ESO_EVENTO EV ON EV.NREVENTO = EC.NREVENTO
                                 AND EV.NMEVENTO = 'S-2221'
               /*VERIFICA EMPRESAS DO MESMO GRUPO - CNPJ RAIZ*/
               JOIN ESTRUTURAH EVH ON EVH.NRORG = EC.NRORG
                                  AND EVH.NRESTRUTURAM = EC.NRESTRUTURAM
                                  AND SUBSTR(REPLACE(REPLACE(REPLACE(EVH.CDCNPJESTRUT,'.'),'/'),'-'),1,8) = SUBSTR(REPLACE(REPLACE(REPLACE(:P_CDCNPJEMPREGADOR,'.'),'/'),'-'),1,8)
                                  AND EVH.DTMESCOMPETENC = (SELECT MAX(H1.DTMESCOMPETENC) FROM ESTRUTURAH H1
                                                             WHERE H1.NRORG = EVH.NRORG
                                                               AND H1.NRESTRUTURAM = EVH.NRESTRUTURAM
                                                               AND H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC)
              WHERE EC.NRORG = :P_NRORG
                AND EC.IDATIVO = 'S' /*QUE ESTIVER ATIVO*/
                AND (EC.IDSTATUS = 'C' OR EC.IDSTATUS = 'A') /*QUE ESTIVER CONCLUIDO*/
                AND EC.NRENVIOEXCL IS NULL /*QUE NAO FOI EXCLUIDO*/
                AND EC.NRTPAMBIENTE = :P_NRTPAMBIENTE
              GROUP BY EC.NRORG, EC.NRCHAVE) ENV ON ENV.NRCHAVE = EFM.NREXAMEFICHAMED
 WHERE EFM.NRORG = :P_NRORG
   AND VH.NRESTRUTLEGAL = :P_NRESTRUTURAM
   AND VM.NRTIPOVINCULOM = 1 /*Somente CLT*/
   AND VM.DTADMISSAOVINC <= LAST_DAY(:P_DTMESCOMPETENC)
   AND (VM.DTRESCISAOVINC IS NULL OR VM.DTRESCISAOVINC >= TRUNC(SYSDATE))
   AND NVL(EFM.DTREALIZACAOEXAME,FM.DTCONCLFICHAMED) BETWEEN :P_DTMESCOMPETENC AND LAST_DAY(:P_DTMESCOMPETENC)
   AND EXA.NRTIPOEXAME = 7 /*Tipo Toxicológico*/
   AND (FM.NRVINCULOM = :P_NRVINCULOM OR :P_NRVINCULOM IS NULL)
   AND (EFM.NREXAMEFICHAMED = :P_NREXAMEFICHAMED OR :P_NREXAMEFICHAMED IS NULL)
   AND (NVL(EFM.DTREALIZACAOEXAME,FM.DTCONCLFICHAMED) >= :P_DTREFNAOPERIOD OR :P_DTREFNAOPERIOD IS NULL)
 ORDER BY 2,5
SQL;
        $params = array(
            "P_NRORG" => $this->hcmParameters->getNrorgTrabalho(),
            "P_DTMESCOMPETENC" => $this->hcmParameters->getDtCompetencia(),
            "P_CDCNPJEMPREGADOR" => isset($params['CDCNPJEMPREGADOR']) ? $params['CDCNPJEMPREGADOR'] : null,
            "P_NRESTRUTURAM" => isset($params['NRESTRUTURAM']) ? $params['NRESTRUTURAM'] : null,
            "P_NRTPAMBIENTE" => isset($params['NRTPAMBIENTE']) ? $params['NRTPAMBIENTE'] : null,
            "P_NRVINCULOM" => isset($params['NRVINCULOM']) ? $params['NRVINCULOM'] : null,
            "P_NREXAMEFICHAMED" => isset($params['NREXAMEFICHAMED']) ? $params['NREXAMEFICHAMED'] : null,
            "P_DTREFNAOPERIOD" => isset($params['DTREFNAOPERIOD']) ? $params['DTREFNAOPERIOD'] : null,
        );
        return $this->connection->fetchAll($sql, $params);
    }

    public function getDadosS2221($parametros) {
        try {
            $dados = $this->getDadosPrincipais($parametros['NRENVIO'], $parametros['NRCHAVE'], $parametros['NRESTRUTURAM']);
            return $dados;
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function getDadosPrincipais($nrenvio, $nrchave, $nrestruturam) {
        $sql = <<<SQL
/*getDadosS2221*/
SELECT EFM.NREXAMEFICHAMED CHAVE,
       'ID'||DECODE(PAR.CDTIPOINSCRICAO,'CNPJ','1','CPF','2','')||RPAD(CASE WHEN PAR.CDTIPOINSCRICAO = 'CPF' OR EMH.CDNATUJURI IN ('1015','1040','1074','1163') THEN PAR.NRINSCRICAOPARC ELSE SUBSTR(PAR.NRINSCRICAOPARC,1,8) END,14,'0')||TO_CHAR(SYSDATE, 'YYYYMMDDHH24MISS')||SUBSTR('00000'||:P_NRENVIO,-5,100) ID,
       --ideEmpregador
       DECODE(PAR.CDTIPOINSCRICAO,'CNPJ','1','CPF','2','') TPINSC,
       CASE WHEN PAR.CDTIPOINSCRICAO = 'CPF' OR EMH.CDNATUJURI IN ('1015','1040','1074','1163') THEN PAR.NRINSCRICAOPARC ELSE SUBSTR(PAR.NRINSCRICAOPARC,1,8) END NRINSC,    
       --ideVinculo
       REPLACE(REPLACE(PH.NRCPFPESSOA,'.'),'-') AS CPFTRAB,  
       NVL(VH.CDMATRICULAESOCIAL, VM.NRVINCULOM) AS matricula,
       --toxicologico
       TO_CHAR(NVL(EFM.DTREALIZACAOEXAME,FM.DTCONCLFICHAMED),'YYYY-MM-DD') AS dtExame,
       REPLACE(REPLACE(REPLACE(EHL.CDCNPJESTRUT,'.'),'/'),'-') AS cnpjLab,
       SUBSTR(EFM.OBSPROCTUSS,1,11) AS codSeqExame,
       PHRM.NMPESSOA AS nmMed,
       PHRM.NRINSCRCONSREG AS nrCRM,
       PHRM.SGUFCONSREG AS ufCRM
  FROM SST_EXAMEFICHAMED EFM
  JOIN SST_FICHAMEDICA FM ON FM.NRORG = EFM.NRORG
                         AND FM.NRFICHAMEDICA = EFM.NRFICHAMEDICA
  /*Dados do Empregado - Vínculo*/
  JOIN GPE_VINCULOM VM ON VM.NRORG = FM.NRORG
                      AND VM.NRVINCULOM = FM.NRVINCULOM
  JOIN GPE_VINCULOH VH ON VH.NRORG = VM.NRORG
                      AND VH.NRVINCULOM = VM.NRVINCULOM
                      AND VH.DTMESCOMPETENC = (SELECT MAX(H1.DTMESCOMPETENC) FROM GPE_VINCULOH H1
                                                WHERE H1.NRVINCULOM = VH.NRVINCULOM
                                                  AND H1.NRORG = VH.NRORG
                                                  AND H1.DTMESCOMPETENC >= ADD_MONTHS(TRUNC(TO_DATE(:P_DTMESCOMPETENC),'YEAR'),-12)
                                                  AND H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC)
  /*Dados do Empregado - Pessoa*/
  JOIN GPE_PESSOA PE ON PE.NRORG = VM.NRORG
                    AND PE.NRPESSOA = VM.NRPESSOA
  JOIN GPE_PESSOAH PH ON PH.NRORG = PE.NRORG
                     AND PH.NRPESSOA = PE.NRPESSOA
                     AND PH.DTMESCOMPETENC = (SELECT MAX(H1.DTMESCOMPETENC) FROM GPE_PESSOAH H1
                                               WHERE H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                 AND H1.NRPESSOA = PH.NRPESSOA
                                                 AND H1.NRORG = PH.NRORG)
  /*Laboratorio*/
  LEFT JOIN ESTRUTURAH EHL ON EHL.NRORG = FM.NRORG
                          AND EHL.NRESTRUTURAM = FM.NRESTRUTURAM
                          AND EHL.DTMESCOMPETENC = (SELECT MAX(H1.DTMESCOMPETENC) FROM ESTRUTURAH H1
                                                     WHERE H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                       AND H1.NRESTRUTURAM = EHL.NRESTRUTURAM
                                                       AND H1.NRORG = EHL.NRORG)
  /*Medico Responsavel*/
  LEFT JOIN GPE_PESSOA PRM ON PRM.NRORG = FM.NRORG
                          AND PRM.NRPARCNEGOCIO = NVL(EFM.NRPARCNEGOCIO,FM.NRPARCNEGOCIO)
  LEFT JOIN GPE_PESSOAH PHRM ON PHRM.NRORG    = PRM.NRORG
                            AND PHRM.NRPESSOA = PRM.NRPESSOA
                            AND PHRM.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_PESSOAH
                                                        WHERE GPE_PESSOAH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                          AND GPE_PESSOAH.NRPESSOA = PHRM.NRPESSOA
                                                          AND GPE_PESSOAH.NRORG = PHRM.NRORG)
  /*Empregador*/
  JOIN ESTRUTURAH EMH ON EMH.NRORG = :P_NRORG
                     AND EMH.NRESTRUTURAM = :P_NRESTRUTURAM
                     AND EMH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM ESTRUTURAH
                                               WHERE ESTRUTURAH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                 AND ESTRUTURAH.NRESTRUTURAM = EMH.NRESTRUTURAM
                                                 AND ESTRUTURAH.NRORG = EMH.NRORG)
  JOIN PARCNEGOCIO PAR ON PAR.NRPARCNEGOCIO = EMH.NRPARCNEGOCIO
                      AND PAR.NRORG = EMH.NRORG
 WHERE EFM.NRORG = :P_NRORG
   AND EFM.NREXAMEFICHAMED = :P_NRCHAVE
SQL;
        $params = array(
            "P_NRORG" => $this->hcmParameters->getNrorgTrabalho(),
            "P_DTMESCOMPETENC" => $this->hcmParameters->getDtCompetencia(),
            "P_NRENVIO" => $nrenvio,
            "P_NRCHAVE" => $nrchave,
            "P_NRESTRUTURAM" => $nrestruturam
        );
        return $this->connection->fetchAssoc($sql, $params);
    }

}