/******************************************************************************/
/* Sistemas Integrados TECNISA                                                */
/*                                                                            */
/* Compras e Suprimentos                                                      */
/*                                                                            */
/******************************************************************************/
/*                                                                            */
/* Nome: PRECOS_TO_FORN                                                       */
/*                                                                            */
/* Descrio:                                                                 */
/*   Carrega os precos para um fornecedor informado.                          */
/*   Carrega tambm os dados necessrios para o clculo do preo econmico    */
/* Criao:                                                                   */
/*    12/05/2004 - Joo Fernando dos Reis Corra                              */
/*                                                                            */
/* Manutenes:                                                               */
/*   /  /   -                                                                 */
/*                                                                            */
/******************************************************************************/

IF EXISTS (SELECT * FROM sysobjects O, sysusers U
          WHERE
            o.name = 'PRECOS_TO_FORN'  and
            type = 'P'                  and
            u.uid = o.uid               and
            u.name = USER)

   DROP PROCEDURE PRECOS_TO_FORN;
GO

CREATE PROCEDURE PRECOS_TO_FORN
(
   @P_CDFILIAL     VARCHAR(4),
   @P_CDFILIPROC   VARCHAR(4),
   @P_NRCOLEPRECO  VARCHAR(10),
   @P_CDREGIAO     VARCHAR(3),
   @P_CDFORNECED   VARCHAR(21),
   @P_CDPRODUTO    VARCHAR(10),
   @P_CDMARCA      VARCHAR(5),
   @P_IDATUREG     VARCHAR(1),
   @P_IDATUFORN    VARCHAR(1)
)
AS
   DECLARE @V_CDEMPRESA  VARCHAR(2);
   DECLARE @V_CDINSCMUNI VARCHAR(16);
   DECLARE @V_CDINSCESTA VARCHAR(16);

   DECLARE @V_DTCOLEPREC DATETIME;
   DECLARE @V_COUNTAUX   NUMERIC;

   DECLARE @V_FORN_REGIAO_CDREGIAO   VARCHAR(3);
   DECLARE @V_FORN_REGIAO_CDFORNECED VARCHAR(21);

   DECLARE @V_PRECOS_CDPRODUTO     VARCHAR(10);
   DECLARE @V_PRECOS_CDARVPROD     VARCHAR(20);
   DECLARE @V_PRECOS_CDMARCA       VARCHAR(5);
   DECLARE @V_PRECOS_VRPRECUNIITM  NUMERIC(15, 5);
   DECLARE @V_PRECOS_CDFORMPGTO    VARCHAR(2);
   DECLARE @V_PRECOS_IDTPFOPGTCOL  VARCHAR(1);
   DECLARE @V_PRECOS_NRPARCPGTCOL  NUMERIC(2);
   DECLARE @V_PRECOS_NRDIAPGFXCOL  NUMERIC(2);
   DECLARE @V_PRECOS_IDUTDIAPGCOL  VARCHAR(1);
   DECLARE @V_NRSEQPRELIST         VARCHAR(10);
   DECLARE @V_IDPRECALT            VARCHAR(1);
BEGIN
   SELECT
      @V_NRSEQPRELIST = MAX(NRSEQPRELIST),
      @V_IDPRECALT    = MAX(IDPRECALT)
   FROM
      ITEMCOLE
   WHERE ITEMCOLE.CDFILIAL    = @P_CDFILIAL
     AND ITEMCOLE.NRCOLEPRECO = @P_NRCOLEPRECO;

   DECLARE C_FORN_REGIAO CURSOR FOR
       SELECT DISTINCT
          CDREGIAO, CDFORNECED
       FROM
          ITEMCOLE
       WHERE
          ITEMCOLE.CDFILIAL    = @P_CDFILIAL    AND
          ITEMCOLE.NRCOLEPRECO = @P_NRCOLEPRECO AND
          ITEMCOLE.CDREGIAO    = CASE @P_IDATUREG WHEN 'T' THEN ITEMCOLE.CDREGIAO ELSE @P_CDREGIAO END AND
          ITEMCOLE.CDFORNECED  <> @P_CDFORNECED AND
          EXISTS (SELECT 1
                  FROM PRODFORNCOMP P
                  WHERE P.CDFORNECED  = ITEMCOLE.CDFORNECED
                    AND P.CDPRODCOMPL = @P_CDPRODUTO) AND
          NOT EXISTS(SELECT
                        CDFORNECED,
                        CDREGIAO
                     FROM
                        REFOREG
                     WHERE
                        REFOREG.CDFORNECED = ITEMCOLE.CDFORNECED AND
                        REFOREG.CDREGIAO   = ITEMCOLE.CDREGIAO);
   OPEN C_FORN_REGIAO;
   FETCH NEXT FROM C_FORN_REGIAO INTO @V_FORN_REGIAO_CDREGIAO, @V_FORN_REGIAO_CDFORNECED;
   WHILE (@@FETCH_STATUS = 0) BEGIN
      SET @V_COUNTAUX = 0;
      SELECT
         @V_COUNTAUX = COUNT(*)
      FROM
         ITEMCOLE
      WHERE
         CDFILIAL    = @P_CDFILIAL    AND
         NRCOLEPRECO = @P_NRCOLEPRECO AND
         CDPRODUTO   = @P_CDPRODUTO   AND
         CDMARCA     = @P_CDMARCA     AND
         CDFORNECED  = @V_FORN_REGIAO_CDFORNECED AND
         CDREGIAO    = @V_FORN_REGIAO_CDREGIAO;


/* SE NA REGIAO/FORNECEDOR DE DESTINO NAO HOUVER O PRODUTO DUPLICADO ENTAO ELE SERA INSERIDO */
      IF @V_COUNTAUX = 0 BEGIN
         INSERT INTO ITEMCOLE(CDFILIAL, NRCOLEPRECO, CDPRODUTO, CDMARCA, CDFORNECED,
                              CDREGIAO, DTINIVALITM, DTFINVALITM,
                              VRPRECUNIITM, VRPERDESITM, IDITMCOTPREC,
                              CDTIPCOB, CDSEGMARCCOL, IDTIPFREITM, NRPZOENTCOL,
                              IDITMLISPREC, CDFORMPGTO, IDTPFOPGTCOL, NRPARCPGTCOL,
                              NRDIAPGFXCOL, IDUTDIAPGCOL, QTACMPITMCOL,
                              DSPZOPGTCONC, IDATUREGIINC, IDATUFORNINC, NRSEQPRELIST,
                              IDPRECALT)
         SELECT ITEMCOLE.CDFILIAL, NRCOLEPRECO, CDPRODUTO, CDMARCA, @V_FORN_REGIAO_CDFORNECED,
                @V_FORN_REGIAO_CDREGIAO, DTINIVALITM, DTFINVALITM,
                VRPRECUNIITM, VRPERDESITM, 'S',
                CDTIPCOB, CDSEGMARCCOL, IDTIPFREITM, NRPZOENTCOL,
                IDITMLISPREC, CDFORMPGTO, IDTPFOPGTCOL, NRPARCPGTCOL,
                NRDIAPGFXCOL, IDUTDIAPGCOL, QTACMPITMCOL,
                DSPZOPGTCONC, @P_IDATUREG, @P_IDATUFORN,
                @V_NRSEQPRELIST, @V_IDPRECALT
         FROM ITEMCOLE,
              FILIAL
         WHERE
            ITEMCOLE.CDFILIAL    = @P_CDFILIAL    AND
            NRCOLEPRECO          = @P_NRCOLEPRECO AND
            CDFORNECED           = @P_CDFORNECED  AND
            CDPRODUTO            = @P_CDPRODUTO   AND
            CDMARCA              = @P_CDMARCA     AND
            ITEMCOLE.CDREGIAO    = @P_CDREGIAO    AND
            FILIAL.CDFILIAL      = @P_CDFILIPROC AND
            ISNULL(FILIAL.IDFUNCAOFILI, 'O') = 'O';

         /* O select que aqui estava, fazendo union all
            com o select acima, foi retirado pois estava causando
            erro interno. Ele no tem nenhuma serventia j que
            tratava os conceitos de filial regional e corporativo
            que alm de no estarem bem definidos, tambm no esto
            em uso e so exclusivos de um cliente */

         INSERT INTO PZOPGTITMCOL
         (CDFILIAL, NRCOLEPRECO,  CDFORNECED,   CDPRODUTO,
          CDMARCA,  NRDIASPGTCOL, VRPERCPGTCOL, CDREGIAO)
         SELECT
          PZOPGTITMCOL.CDFILIAL, PZOPGTITMCOL.NRCOLEPRECO, @V_FORN_REGIAO_CDFORNECED,
          PZOPGTITMCOL.CDPRODUTO,PZOPGTITMCOL.CDMARCA, PZOPGTITMCOL.NRDIASPGTCOL,
          PZOPGTITMCOL.VRPERCPGTCOL, @V_FORN_REGIAO_CDREGIAO
         FROM
          PZOPGTITMCOL
         WHERE
          PZOPGTITMCOL.CDREGIAO    = @P_CDREGIAO    AND
          PZOPGTITMCOL.CDFORNECED  = @P_CDFORNECED  AND

          PZOPGTITMCOL.CDFILIAL    = @P_CDFILIAL    AND
          PZOPGTITMCOL.NRCOLEPRECO = @P_NRCOLEPRECO AND
          PZOPGTITMCOL.CDPRODUTO   = @P_CDPRODUTO   AND
          PZOPGTITMCOL.CDMARCA     = @P_CDMARCA;
      END;

      SET @V_DTCOLEPREC = NULL;
      SELECT  @V_DTCOLEPREC = MAX(DTCOLEPREC)
      FROM COLETAPR
      WHERE
         CDFILIAL    = @P_CDFILIAL AND
         NRCOLEPRECO = @P_NRCOLEPRECO;

      DECLARE C_PRECOS CURSOR FOR
            SELECT
              ITEMCOLE.CDPRODUTO,
              PRODUTO.CDARVPROD,
              ITEMCOLE.CDMARCA,
              ITEMCOLE.VRPRECUNIITM,
              ITEMCOLE.CDFORMPGTO,
              ITEMCOLE.IDTPFOPGTCOL,
              ITEMCOLE.NRPARCPGTCOL,
              ITEMCOLE.NRDIAPGFXCOL,
              ITEMCOLE.IDUTDIAPGCOL
            FROM
              ITEMCOLE, PRODUTO
            WHERE
              ITEMCOLE.CDPRODUTO   = PRODUTO.CDPRODUTO AND
              ITEMCOLE.CDFILIAL    = @P_CDFILIAL        AND
              ITEMCOLE.NRCOLEPRECO = @P_NRCOLEPRECO     AND
              ITEMCOLE.CDREGIAO    = @P_CDREGIAO        AND
              ITEMCOLE.CDFORNECED  = @P_CDFORNECED      AND
              ITEMCOLE.CDPRODUTO   = @P_CDPRODUTO       AND
              ITEMCOLE.CDMARCA     = @P_CDMARCA         AND
              NOT EXISTS
               (SELECT
                  REFOREG.CDREGIAO
                FROM
                  REFOREG
                WHERE
                  REFOREG.CDREGIAO    = ITEMCOLE.CDREGIAO AND
                  REFOREG.CDFORNECED  = ITEMCOLE.CDFORNECED);
      OPEN C_PRECOS;
      FETCH NEXT FROM C_PRECOS INTO @V_PRECOS_CDPRODUTO,
                                    @V_PRECOS_CDARVPROD,
                                    @V_PRECOS_CDMARCA,
                                    @V_PRECOS_VRPRECUNIITM,
                                    @V_PRECOS_CDFORMPGTO,
                                    @V_PRECOS_IDTPFOPGTCOL,
                                    @V_PRECOS_NRPARCPGTCOL,
                                    @V_PRECOS_NRDIAPGFXCOL,
                                    @V_PRECOS_IDUTDIAPGCOL;
      WHILE (@@FETCH_STATUS = 0) BEGIN

/* CUSOR QUE RETORNARA TODOS ITENS QUE SERAO DUPLICADOS */

         UPDATE ITEMCOLE
         SET VRPRECUNIITM = @V_PRECOS_VRPRECUNIITM,
             IDITMCOTPREC = 'S',
             IDATUREGIALT = @P_IDATUREG,
             IDATUFORNALT = @P_IDATUFORN
         WHERE
           CDFILIAL    = @P_CDFILIAL     AND
           NRCOLEPRECO = @P_NRCOLEPRECO  AND
           CDREGIAO    = @V_FORN_REGIAO_CDREGIAO AND
           CDFORNECED  = @V_FORN_REGIAO_CDFORNECED AND
           CDPRODUTO   = @V_PRECOS_CDPRODUTO  AND
           CDMARCA     = @V_PRECOS_CDMARCA;

/*###############################  IMPOSTO ######################################*/

           SET @V_CDEMPRESA = NULL;
           SELECT
              @V_CDEMPRESA = MAX(EMITENTEAF.CDEMPRESA)
           FROM
              EMITENTEAF, PRODUTO
           WHERE
              EMITENTEAF.CDPRODUTO = PRODUTO.CDPRODUTO AND
              EMITENTEAF.CDFILIAL = @P_CDFILIAL AND
              @V_PRECOS_CDARVPROD LIKE PRODUTO.CDARVPROD + '%';

           IF @V_CDEMPRESA IS NULL BEGIN
              SET @V_CDEMPRESA = NULL;
              SELECT
                 @V_CDEMPRESA = MAX(PARCMPSUP.CDEMPRESA)
              FROM
                 PARCMPSUP
              WHERE
                 PARCMPSUP.CDFILIAL = @P_CDFILIAL;
           END;

           SET @V_CDEMPRESA  = NULL;
           SET @V_CDINSCMUNI = NULL;
           SET @V_CDINSCESTA = NULL;
           SELECT
              @V_CDEMPRESA  = MAX(EMPRESA.CDEMPRESA),
              @V_CDINSCMUNI = MIN(INSCRMUNICIP.CDINSCMUNI),
              @V_CDINSCESTA = MIN(INSCRESTAD.CDINSCESTA)
           FROM
              EMPRESA,
              INSCRMUNICIP,
              INSCRESTAD
           WHERE
              EMPRESA.CDEMPRESA = @V_CDEMPRESA           AND
              EMPRESA.CDEMPRESA = INSCRMUNICIP.CDEMPRESA AND
              EMPRESA.CDEMPRESA = INSCRESTAD.CDEMPRESA;

         EXEC IMPOSTO_ITEMCOL @V_CDEMPRESA,      @V_CDINSCESTA,             @V_FORN_REGIAO_CDFORNECED, @P_CDFILIAL,
                              @P_NRCOLEPRECO,    @V_FORN_REGIAO_CDFORNECED, @V_FORN_REGIAO_CDREGIAO,   @V_PRECOS_CDPRODUTO,
                              @V_PRECOS_CDMARCA, @V_DTCOLEPREC;
         UPDATE ITEMCOLE
         SET VRPRECUNIITM = VRPRECUNIITM
         WHERE
           CDFILIAL    = @P_CDFILIAL     AND
           NRCOLEPRECO = @P_NRCOLEPRECO  AND
           CDREGIAO    = @V_FORN_REGIAO_CDREGIAO AND
           CDFORNECED  = @V_FORN_REGIAO_CDFORNECED AND
           CDPRODUTO   = @V_PRECOS_CDPRODUTO  AND
           CDMARCA     = @V_PRECOS_CDMARCA;

         FETCH NEXT FROM C_PRECOS INTO @V_PRECOS_CDPRODUTO,
                                       @V_PRECOS_CDARVPROD,
                                       @V_PRECOS_CDMARCA,
                                       @V_PRECOS_VRPRECUNIITM,
                                       @V_PRECOS_CDFORMPGTO,
                                       @V_PRECOS_IDTPFOPGTCOL,
                                       @V_PRECOS_NRPARCPGTCOL,
                                       @V_PRECOS_NRDIAPGFXCOL,
                                       @V_PRECOS_IDUTDIAPGCOL;
      END;
      CLOSE C_PRECOS;
      DEALLOCATE C_PRECOS;
      FETCH NEXT FROM C_FORN_REGIAO INTO @V_FORN_REGIAO_CDREGIAO, @V_FORN_REGIAO_CDFORNECED;
   END;
   CLOSE C_FORN_REGIAO;
   DEALLOCATE C_FORN_REGIAO;
END;
GO

-- VERSAO --
EXEC ATUVEROBJ 'PRECOS_TO_FORN','6.25.xxx';
GO
